(function () {

var _ = require('underscore');
var request = require('request');
var fs = require('fs');
var Encoder = require('node-html-encoder').Encoder;
var encoder = new Encoder('entity');

var GYM_URL = 'https://www1.recreation.rutgers.edu/facilities/json.asp?StartDate=';

// ## gym
// The main exported function. Sets up the recreation scraper and begins
// scraping.  All supporting methods are attached to gym.
function gym (config, errback) {
  var log = config.log;
  var timer;
  var statusData = {};

  var tmpPath = config.paths.temp + '/gyms.txt';
  var outPath = config.paths.output + '/gyms.txt';

  function scrape () {
    var today = new Date();
    today.setTime(today.getTime() - (1000 * 60 * 60 * 24 * 30));
    var url = GYM_URL + 
      today.getFullYear() + '/' + (today.getMonth()+1) + '/' + today.getDate();
    log.info("gym: requesting " + url);

    request(url, function (er, res, body) {
      if (er) return log.error("gym: " + er);
      var data = gym.fixer(body);
      if (data instanceof Error) return log.error(data);

      var str = JSON.stringify(data);

      fs.writeFile(tmpPath, str, function (err) {
        if (err) return log.error("gym: " + err);

        fs.rename(tmpPath, outPath, function (err) { 
          if (err) return log.error("gym: " + err);

          log.info('gym: completed scrape');
          statusData = {last: new Date()};
        });
      });
    });
  }

  function status (callback) {
    callback(null, statusData);
  }

  function shutdown (done) {
    clearInterval(timer);
    done();
  }

  function start () {
    log.info("gym: starting");
    timer = setInterval(scrape, Number(config.modules.gym.interval) * 1000 * 60);
    scrape();
  }

  return {
    start: start,
    status: status,
    shutdown: shutdown
  };
}

gym.stripHtml = function (str) {
  return encoder.htmlDecode(str)
    .replace(/(<([^>]+)>)/ig,"")      // strip html tags
    .replace(/\r\n\r\n/g, ' \n\n')    // replace double \r\n with \n
    .replace(/\r\n/g, ' ')            // remove remaining newlines
    .replace(/&amp;/g, '&')           // these arent getting decoded properly
    .replace(/^ ?\n\n/, '');          // if it begins with \n\n remove that
};

// ## fixer
// Fixes the 'json' we get from recreation to be valid json.  Returns the
// parsed json, or an error instance if there was an error.
gym.fixer = function (input) {
  // Keys we'll be fixing. Each of these is specified multiple times in a hash
  // in the input 'json'.

  var parsed;
  try { parsed = JSON.parse(input); } catch (e) { return e; }

  // Now the data will parse, but we still need to fix the objects we fixed
  // above with the regex; they should be arrays but they are objects instead.
  // We applied sequential numbering regardless of which set of meetingareas
  // and schedules we were considering, so now we'll have to fix this as well.

  // The following looks confusing, but it's just a set of nested reductions.

  var campuses = _.reduce(parsed.campuses, function (memo, campus) {
    delete campus["CampusID"];
    delete campus["CampusBrief"];
    var title = campus.CampusTitle;
    delete campus.CampusTitle;

    campus = _.reduce(campus.facilities, function (memo, fac) {
      var areaCount = 0;

      delete fac["FacilityRentalInfo"];
      delete fac["FacilityID"];
      var facTitle = fac["FacilityTitle"];
      delete fac["FacilityTitle"];

      fac.FacilityBody = gym.stripHtml(fac.FacilityBody);
      fac.FacilityBrief = gym.stripHtml(fac.FacilityBrief);

      fac.meetingareas = _.reduce(fac.meetingareas, function (memo, area) {
        var areaTitle = area['MeetingAreaName'];
        delete area.MeetingAreaName;
        delete area.MeetingAreaID;

        area = _.reduce(area.schedules, function (memo, sched) {
          memo[sched.ScheduleDate] = sched.ScheduleText
            .replace(/ \- /g, "-")
            .replace(/\-/g, " - ");
          return memo;
        }, {});

        memo[areaTitle] = area;
        return memo;
      }, {});

      memo[facTitle] = fac;
      return memo;
    }, {});

    memo[title] = campus;
    return memo;
  }, {});

  return campuses;
};

exports.module = gym;

}());
